 title  "2WireLCD - Write 'Hello' to an LCD using 2 Wires"
;
;  This Code Sends the data to the LCD in 4 Bit mode and
;   writes the string: "Hello".
;
;  The data is shifted out to a 74LS174 with The High Order Bit being a "Gate"
;   Bit connected to the LCD's "E" line.  The next Highest Bit goes to the
;   LCD's "RS" Bit.  The lowest Bit goes to D4 (and each Bit is higher from
;   there).
;
;
;  Hardware Notes:
;   Reset is tied directly to Vcc and PWRT is Enabled.
;   The PIC is a 16C84 Running at 4 MHz.
;   PortA.0 is the Data Bit
;   PortA.1 is the Clock Bit
;
;
;  Data Being Sent looks like:
;
;          Clear S/R           Shift Data      Strobe
;
;         Ŀ     Ŀ
;                                         
;
;                      ı    ڿ
;  Data                      
;        ĿڿڿڿڿڿĿڿڿڿڿڿ
;  Clock            
;                                               ڿ
;  E     
;
;  Shift Data:
;  Bit 1 - Always High (Gate for "E")
;  Bit 2 - RS Bit
;  Bit 3 - LCD D4
;  Bit 4 - LCD D5
;  Bit 5 - LCD D6
;  Bit 6 - LCD D7
;
;  Myke Predko
;  99.03.13
;
  LIST P=16f84, R=DEC          ;  16F84 Runs at 4 MHz
  errorlevel 0,-305
  INCLUDE "p16f84.inc"


;  Register Usage
 CBLOCK 0x00C                   ;  Start Registers at End of the Values
Dlay                            ;  8 Bit Delay Variable
Temp				;  Temporary Value Used When Sending Out Data
NOTemp				;  Temporary Value to "NybbleOutput"
 ENDC


;  Define Inforation
#DEFINE 	Data 	PORTA,0
#DEFINE 	Clock 	PORTA,1
;
TM_L		equ	15h
TM_H		equ	16h
UNI			equ	17h
;  Macros
ClockStrobe MACRO		;  Strobe the Data Bit
  bsf	 Clock
  bcf	 Clock
 ENDM

EStrobe MACRO			;  Strobe the "E" Bit
  bsf	 Data
  bcf	 Data
 ENDM


 PAGE
 __CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF
;  Note that the WatchDog Timer is OFF
;  Demo Code, Loop Forever Toggling PA0 (Flashing the LED)

  org    0

  clrf   PORTA

  movlw  0x01C          ;  Enable PA0 & PA1 for Output
  bsf    STATUS, RP0
  movwf  TRISA ^ 0x080
  bcf    STATUS, RP0

  call   Dlay5          ;  Wait 20 msecs before Reset
  call   Dlay5
  call   Dlay5
  call   Dlay5

  bcf    STATUS, C      ;  Clear Carry (Instruction Out)
  movlw  0x03           ;  Reset Command
  call   NybbleOut      ;  Send the Nybble
  call	 Dlay5			;  Wait 5 msecs before Sending Again

  EStrobe
  call	 Dlay160		;  Wait 160 usecs before Sending the Third Time

  EStrobe
  call	 Dlay160		;  Wait 160 usecs before Sending the Third Time

  bcf    STATUS, C              
  movlw  0x02           ;  Set 4 Bit Mode
  call   NybbleOut              
  call	 Dlay160		

  movlw	 0x028			;  Note that it is a 2 Line Display
  call	 SendINS

  movlw	 0x008			;  Turn off the Display
  call	 SendINS

  movlw	 0x001			;  Clear the Display RAM
  call	 SendINS
  call	 Dlay5			;  Note, Can take up to 4.1 msecs

  movlw	 0x006			;  Enable Cursor Move Direction
  call	 SendINS

  movlw	 0x00C			;  Turn the LCD Back On
  call	 SendINS

  clrf	 FSR			;  Output the Message
OutLoop
  movf	 FSR, w			;  Get the Offset to Output
  incf	 FSR
  call	 Message
  iorlw	 0				;  At the End of the Message?
  btfsc	 STATUS, Z
   goto	 $+3			;  Yes - Equal to Zero
  call	 SendCHAR		;  Output the ASCII Character
  goto	 OutLoop
;******************************************************
	MOVLW	0C0H
	CALL	SendINS
  clrf	 FSR			;  Output the Message
OutLoop2
  movf	 FSR, w			;  Get the Offset to Output
  incf	 FSR
  call	 Message2
  iorlw	 0				;  At the End of the Message?
  btfsc	 STATUS, Z
   goto	 Loop			;  Yes - Equal to Zero
  call	 SendCHAR		;  Output the ASCII Character
  goto	 OutLoop2
;******************************************************
Loop					;  Loop Forever when Done
	MOVLW	18H
	CALL	SendINS
	CALL	ISTOTA
	GOTO	$-3
  goto   Loop


;  Subroutines
Message					;  Message to Output
  addwf	 PCL			;  Output the Characters
  dt	 "**Funguje to super**", 0
Message2					;  Message to Output
  addwf	 PCL			;  Output the Characters
  dt	 "******Skusim dva riadky******", 0

SendCHAR				;  Send the Character to the LCD
  movwf	 Temp			;  Save the Temporary Value

  swapf	 Temp, w		;  Send the High Nybble
  bsf	 STATUS, C		;  RS = 1
  call	 NybbleOut

  movf	 Temp, w		;  Send the Low Nybble
  bsf	 STATUS, C
  call	 NybbleOut

  return

SendINS					;  Send the Instruction to the LCD
  movwf	 Temp			;  Save the Temporary Value

  swapf	 Temp, w		;  Send the High Nybble
  bcf	 STATUS, C		;  RS = 0
  call	 NybbleOut

  movf	 Temp, w		;  Send the Low Nybble
  bcf	 STATUS, C
  call	 NybbleOut

  return

NybbleOut				;  Send a Nybble to the LCD

  movwf	 NOTemp			;  Save the Nybble to Shift Out
  swapf	 NOTemp			;  Setup to Output to the High Part of the Byte

  movlw	 6				;  Clear the Shift Register
  movwf	 Dlay
NOLoop1
  ClockStrobe
  decfsz Dlay
   goto	 NOLoop1
   MOVLW	5
   MOVWF	Dlay
   bsf	 Data			;  Put out the Gate Bit
  ClockStrobe
 ; bcf	 Data			;  Put out the RS Bit
 ; rlf	 PORTA
 ; ClockStrobe
 ; movlw	 4				;  Now, Shift out the Data
 ; movwf	 Dlay
NOLoop2
	BCF		Data
	BTFSC	STATUS,C
	BSF		Data
 ; rlf	 NOTemp			;  Shift Through the Nybble to Output
 ; bcf	 Data			;  Clear the Data Bit (which is the Clock)
 ; rlf	 PORTA			;  Shift the Carry into the Shift Register
  ClockStrobe
  RLF		NOTemp
  decfsz Dlay
   goto	 NOLoop2

  EStrobe				;  Strobe out the LCD Data

  return


Dlay160                         ;  Delay 160 usecs

  movlw  256 - ( 160 / 4 )      ;  Loop Until Carry Set
  addlw  1
  btfss  STATUS, C
   goto  $-2

  return

Dlay5                           ;  Delay 5 msecs

  movlw  4                      ;  Set up the Delay
  movwf  Dlay
  movlw  256 - 0x0E8
  addlw  1
  btfsc  STATUS, Z
   decfsz Dlay
    goto $-3

  return
;dvojbajtova slucka od m.hrona (kte magazin)
;	cislo=(cas-20)/10
;	cas sa zadava v mikrosekundach a vysledok preved na hexa(win kalkulacka)
;	a napln registre TM_H a TM_L ....... a ty go go....
;	rozsah 30(?) az 655 360 mikrosekund
;**********************************************************************
SEKUNDA	CALL	POL_SEK		;CAKANIE JEDNU SEKUNDU
		CALL	POL_SEK
		RETURN
;
ISTOTA	MOVLW	0x1E		;CAKANIE 200 MILISEKUND
		MOVWF	TM_L		;NIZSII HEX
		MOVLW	0x4E		;VYSSI HEX
		MOVWF	TM_H
		GOTO	SLUCKA
;
POL_SEK	MOVLW	0x4E				;CASOVA SLUCKA 500 MILISEKUND
		MOVWF	TM_L				;HEXA NIZSIEHO BAJTU
		MOVLW	0xC3				;HEXA VYSSIEHO BAJTU
		MOVWF	TM_H
		GOTO	SLUCKA
;
C_114MS	MOVLW	086H		;CAKANIE 114 MILISEKUND
		MOVWF	TM_L		;NIZSI HEX
		MOVLW	02CH		;VYSSI HEX
		MOVWF	TM_H
		GOTO	SLUCKA
;
PAUZA	MOVLW	0x55		;PAUZA DLHA POL BITU-889 MIKROSEKUND
		MOVWF	TM_L		;NIZSI HEX
		MOVLW	0x00		;VYSSI HEX
		MOVWF	TM_H
		MOVLW	3
		MOVWF	UNI
		DECFSZ	UNI,F
		GOTO	$-1
		NOP	
		GOTO	SLUCKA
;
;  
;		NOP
;		NOP
SLUCKA	NOP					;SPOLOCNA CAKACIA SLUCKA
		NOP
SLC		MOVF	TM_L,F
		BTFSS	STATUS,Z
		GOTO	SLC_1
		MOVF	TM_H,F
		BTFSC	STATUS,Z
		GOTO	SLC_2
		DECF	TM_H,F
SLC_3	DECF	TM_L,F
		GOTO	SLC
SLC_1	NOP
		GOTO	SLC_3
SLC_2	NOP
		RETURN				;KONIEC CAKANIA

  end
